// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "appwindow.h"
#include "messages.h"
#include "common.h"
#include <storage/file.h>

using namespace os;
using namespace std;

AppWindow :: AppWindow( const Rect& cFrame ) : Window( cFrame, "main_window", "SLogic" )
{	
	// Create tools
	m_ToolArrow = new ToolArrow();
	m_ToolConnect = new ToolConnect();
	m_ToolHand = new ToolHand();

	// Set current tool
	m_CurrentTool = m_ToolArrow;
	m_CurrentTool->Init();

	// Create toolbar
	m_Toolbar = new ToolBar( Rect(), "toolbar" );

	Message* msg = new Message( MSG_TOOLBAR_TOOL );
	msg->AddPointer( "tool", m_ToolArrow );
	m_Toolbar->AddButton( "", "Arrow", LoadImage( "tool_arrow.png" ), msg );

	msg = new Message( MSG_TOOLBAR_TOOL );
	msg->AddPointer( "tool", m_ToolConnect );
	m_Toolbar->AddButton( "", "Connector", LoadImage( "tool_connect.png" ), msg );

	msg = new Message( MSG_TOOLBAR_TOOL );
	msg->AddPointer( "tool", m_ToolHand );
	m_Toolbar->AddButton( "", "Hand", LoadImage( "tool_hand.png" ), msg );

	AddChild( m_Toolbar );

	// Create sidebars
	m_ComponentListSidebar = new CompListSideBar(  Rect(), CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT );
	AddChild( m_ComponentListSidebar );

	// Set current sidebar
	m_CurrentSideBar = m_ComponentListSidebar;

	// Create component draw
	m_ComponentDraw = new CompDraw( Rect() );
	AddChild( m_ComponentDraw );

	// Fix layout
	Layout( true );

	// Init variables
	AddTimer( this, SIMULATION_TIMER, 10000, false );
}

AppWindow :: ~AppWindow()
{
	delete m_ToolHand;	
	delete m_ToolArrow;	
	delete m_ToolConnect;
	delete m_Toolbar;
	delete m_ComponentListSidebar;
	delete m_ComponentDraw;
}

void AppWindow :: Layout( bool force )
{
	float offset = 4.0f;

	// Get size of the window
	Rect cBounds = GetBounds();

	// Calculate toolbar
	Rect cToolbarFrame( cBounds.left, cBounds.top, cBounds.right, cBounds.top + m_Toolbar->GetPreferredSize( false ).y );
	m_Toolbar->SetFrame( cToolbarFrame );
	cBounds.top = cToolbarFrame.bottom + 1.0f;

	// Calculate sidbar
	if( m_CurrentSideBar != NULL && ( m_CurrentSideBar->IsVisible() || force ) )
	{
		Rect cSidebarFrame = cBounds;
		cSidebarFrame.right = cSidebarFrame.left + m_CurrentSideBar->GetPreferredSize( false ).x;
		m_CurrentSideBar->SetFrame( cSidebarFrame );
		cBounds.left = cSidebarFrame.right + 1.0f;
	}

	m_ComponentDraw->SetFrame( cBounds );
}

void AppWindow :: FrameSized( const Point& delta )
{
	Window::FrameSized( delta );

	Layout();
}

void AppWindow :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{		
		case MSG_TOOLBAR_TOOL:
		{
			if( m_CurrentTool != NULL )
				m_CurrentTool->Deinit();

			m_ComponentDraw->DeselectAll();

			void* p;

			if( pcMessage->FindPointer( "tool", (void**) &p ) == EOK )
			{
				m_CurrentTool = (Tool*) p;
				m_CurrentTool->Init();
			}
			break;
		}	
		case MSG_SIDEBAR_HIDE:	
		{
			m_CurrentSideBar->Hide();
			m_CurrentSideBar = NULL;
			Layout();
			break;
		}
		default:
		{
			Window::HandleMessage( pcMessage );
			break;
		}
	}
}

void AppWindow :: TimerTick( int nID )
{
	m_ComponentDraw->UpdateSimulation();
}

bool AppWindow :: OkToQuit( void )
{
	return true;
}

void AppWindow :: RedrawComponent( Component* comp )
{
	if( m_ComponentDraw != NULL )
		m_ComponentDraw->RedrawComponent( comp );
}

