// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __COMPDRAW_H_
#define __COMPDRAW_H_

#include <vector>
#include <atheos/time.h>
#include <gui/view.h>
#include <gui/rect.h>
#include <gui/point.h>

#include "component.h"
#include "tool_arrow.h"
#include "tool_connect.h"
#include "tool_hand.h"
#include "line_segment.h"

class CompDraw : public os::View
{
public:
	friend class ToolArrow;
	friend class ToolConnect;
	friend class ToolHand;

	CompDraw( const os::Rect& cFrame, uint32 nResizeMask = os::CF_FOLLOW_LEFT | os::CF_FOLLOW_TOP );
	~CompDraw();

	void MouseUp( const os::Point& cPosition, uint32 nButtons, os::Message* pcData );
	void MouseDown( const os::Point& cPosition, uint32 nButtons );
	void MouseMove( const os::Point& cPosition, int nCode, uint32 nButtons, os::Message* pcData );

	virtual void Paint( const os::Rect& cUpdate );

	void AddComponent( Component* comp );

	int HitTest( const os::Point& cPosition, os::Point bound = os::Point( 0, 0 ) );
	void DeselectAll();
	void Select( uint nComp, bool bSelected );
	void Select( os::Point cStart, os::Point cEnd, bool bKeepSelection );

	os::Point Snap( os::Point pos );

	void DrawDigitalLine( os::Point start, os::Point end );
	
	void AddLineSegment( LineSegment* seg ) { m_ListOfLineSegments.push_back( seg ); };

	void RedrawComponent( Component* comp );

	void UpdateSimulation();

	int GetSimulationTime() { return m_SimulationTime; };

private:
	bool m_SnapMode;
	float m_SnapX;
	float m_SnapY;

	std::vector< Component* > m_ListOfComponents;
	std::vector< LineSegment* > m_ListOfLineSegments;

	int m_SimulationTime;
};

#endif
