// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <util/application.h>
#include "tool_connect.h"
#include "component.h"
#include "compdraw.h"
#include "pin_io.h"
#include "line_segment.h"

using namespace os;

uint8 g_anMouseImg[] = {
	0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	2,1,2,1,2,0,0,2,0,0,2,1,2,1,2,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,2,0,0,0,0,0,0,0
	};

ToolConnect :: ToolConnect() : Tool( 1 )
{
}

ToolConnect :: ~ToolConnect()
{
}

void ToolConnect :: Init()
{
	m_bMouseDown = false;
	m_Dragging = false;
	m_HitDraw = false;
	m_HitPin_1 = NULL;
	m_HitPin_2 = NULL;
}

void ToolConnect :: SetCursor()
{
	Application::GetInstance()->PushCursor( MPTR_MONO, g_anMouseImg, 15, 15, IPoint( 8, 8 ) );
}

void ToolConnect :: MouseUp( CompDraw* draw, const Point& cPosition, uint32 nButtons )
{
	m_bMouseDown = false;
	m_Dragging = false;

	// Remove old line
	if( m_Dragging )
	{
		draw->SetDrawingMode( os::DM_INVERT );
		draw->DrawDigitalLine( m_cStartPoint, m_cLastPoint );
		draw->Flush();

		m_Dragging = false;
	}

	// Remove old hit rectangle and add line segment
	if( m_HitDraw )
	{
		draw->SetDrawingMode( os::DM_INVERT );
		draw->DrawFrame( Rect( m_cLastHitPoint - Point( 5, 5 ), m_cLastHitPoint + Point( 5, 5 ) ), os::FRAME_THIN | os::FRAME_TRANSPARENT );
		draw->Flush();

		m_HitDraw = false;

		if( m_HitPin_1 != NULL && m_HitPin_2 != NULL )
		{
			draw->AddLineSegment( new LineSegment( m_cStartPoint, m_cLastPoint ) );
			m_HitPin_1->ConnectTo( m_HitPin_2 );
			m_HitPin_2->ConnectTo( m_HitPin_1 );
		}
	}
}

void ToolConnect :: MouseDown( CompDraw* draw, const Point& cPosition, uint32 nButtons )
{
	if( m_HitDraw )
	{
		m_bMouseDown = true;
		m_Dragging = false;
		m_cStartPoint = m_cLastHitPoint;
	}
}

void ToolConnect :: MouseMove( CompDraw* draw, const Point& cP, uint32 nButtons )
{
	int nComp;
	Point cPosition = cP;
	bool HitIO = false;

	// Remove old hit rectangle
	if( m_HitDraw )
	{
		draw->SetDrawingMode( os::DM_INVERT );
		draw->DrawFrame( Rect( m_cLastHitPoint - Point( 5, 5 ), m_cLastHitPoint + Point( 5, 5 ) ), os::FRAME_THIN | os::FRAME_TRANSPARENT );
		draw->Flush();

		m_HitDraw = false;
	}

	// Check if we hit a input pin or a output pin
	if( ( nComp = draw->HitTest( cPosition, Point( 5, 5 ) ) ) >= 0 )
	{
		PinIO* hit = draw->m_ListOfComponents[ nComp ]->HitCheckPin( cPosition );

		// Draw hit rectangle
		if( hit != NULL )
		{
			cPosition = hit->GetPosition();

			draw->SetDrawingMode( os::DM_INVERT );
			draw->DrawFrame( Rect( cPosition - Point( 5, 5 ), cPosition + Point( 5, 5 ) ), os::FRAME_THIN | os::FRAME_TRANSPARENT );
			draw->Flush();

			m_cLastHitPoint = cPosition;
			m_HitDraw = true;
		}

		if( m_bMouseDown )
			m_HitPin_2 = hit;
		else
			m_HitPin_1 = hit;
		
	}

	if( m_bMouseDown )
	{
		if( m_Dragging )
		{
			draw->SetDrawingMode( os::DM_INVERT );
			draw->DrawDigitalLine( m_cStartPoint, m_cLastPoint );

			m_Dragging = false;
		}

		// Draw new line
		draw->SetDrawingMode( os::DM_INVERT );
		draw->DrawDigitalLine( m_cStartPoint, cPosition );

		m_cLastPoint = cPosition;
		m_Dragging = true;

		draw->Flush();
	}
}
